/*
Copyright The KCP Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"context"
	"time"

	logicalcluster "github.com/kcp-dev/logicalcluster"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"

	v1alpha1 "github.com/kcp-dev/kcp/pkg/apis/scheduling/v1alpha1"
	scheme "github.com/kcp-dev/kcp/pkg/client/clientset/versioned/scheme"
)

// LocationsGetter has a method to return a LocationInterface.
// A group's client should implement this interface.
type LocationsGetter interface {
	Locations() LocationInterface
}

// LocationInterface has methods to work with Location resources.
type LocationInterface interface {
	Create(ctx context.Context, location *v1alpha1.Location, opts v1.CreateOptions) (*v1alpha1.Location, error)
	Update(ctx context.Context, location *v1alpha1.Location, opts v1.UpdateOptions) (*v1alpha1.Location, error)
	UpdateStatus(ctx context.Context, location *v1alpha1.Location, opts v1.UpdateOptions) (*v1alpha1.Location, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1alpha1.Location, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1alpha1.LocationList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.Location, err error)
	LocationExpansion
}

// locations implements LocationInterface
type locations struct {
	client  rest.Interface
	cluster logicalcluster.Name
}

// newLocations returns a Locations
func newLocations(c *SchedulingV1alpha1Client) *locations {
	return &locations{
		client:  c.RESTClient(),
		cluster: c.cluster,
	}
}

// Get takes name of the location, and returns the corresponding location object, and an error if there is any.
func (c *locations) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.Location, err error) {
	result = &v1alpha1.Location{}
	err = c.client.Get().
		Cluster(c.cluster).
		Resource("locations").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Locations that match those selectors.
func (c *locations) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.LocationList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.LocationList{}
	err = c.client.Get().
		Cluster(c.cluster).
		Resource("locations").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested locations.
func (c *locations) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Cluster(c.cluster).
		Resource("locations").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a location and creates it.  Returns the server's representation of the location, and an error, if there is any.
func (c *locations) Create(ctx context.Context, location *v1alpha1.Location, opts v1.CreateOptions) (result *v1alpha1.Location, err error) {
	result = &v1alpha1.Location{}
	err = c.client.Post().
		Cluster(c.cluster).
		Resource("locations").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(location).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a location and updates it. Returns the server's representation of the location, and an error, if there is any.
func (c *locations) Update(ctx context.Context, location *v1alpha1.Location, opts v1.UpdateOptions) (result *v1alpha1.Location, err error) {
	result = &v1alpha1.Location{}
	err = c.client.Put().
		Cluster(c.cluster).
		Resource("locations").
		Name(location.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(location).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *locations) UpdateStatus(ctx context.Context, location *v1alpha1.Location, opts v1.UpdateOptions) (result *v1alpha1.Location, err error) {
	result = &v1alpha1.Location{}
	err = c.client.Put().
		Cluster(c.cluster).
		Resource("locations").
		Name(location.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(location).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the location and deletes it. Returns an error if one occurs.
func (c *locations) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Cluster(c.cluster).
		Resource("locations").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *locations) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Cluster(c.cluster).
		Resource("locations").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched location.
func (c *locations) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.Location, err error) {
	result = &v1alpha1.Location{}
	err = c.client.Patch(pt).
		Cluster(c.cluster).
		Resource("locations").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
